from flask import Flask, g, render_template, request
from flask_bootstrap import Bootstrap
from flask_mail import Mail
from flask_sqlalchemy import SQLAlchemy
from flask_login import LoginManager
from flask_wtf.csrf import CSRFProtect, CSRFError
from flask_babel import Babel
from werkzeug.contrib.fixers import ProxyFix
from proxadmin import configure_app

app = Flask(__name__)

configure_app(app)

app.wsgi_app = ProxyFix(app.wsgi_app) #trusting headers behind proxy

db = SQLAlchemy(session_options = { "autoflush": False })
lm = LoginManager()
lm.login_view = 'auth.login'
lm.login_message = 'Login Required.'
lm.session_protection = 'strong'
#lm.session_protection = 'basic'
mail = Mail()
bootstrap = Bootstrap()
#csrf = CSRFProtect()
babel = Babel()

app = Flask(__name__)
app.config.from_object('config')

#if app.debug:
#    pass
#    #toolbar = DebugToolbarExtension(app)
#else:
#    import logging
#    from logging.handlers import RotatingFileHandler
#    file_handler = RotatingFileHandler('app/log/proxadmin.log', 'a', 1 * 1024 * 1024, 10)
#    file_handler.setFormatter(logging.Formatter('%(asctime)s %(levelname)s: %(message)s [in %(pathname)s:%(lineno)d]'))
#    app.logger.setLevel(logging.INFO)
#    file_handler.setLevel(logging.INFO)
#    app.logger.addHandler(file_handler)
#    app.logger.info('Proxadmin started.')

bootstrap.init_app(app)
mail.init_app(app)
db.init_app(app)
lm.init_app(app)
babel.init_app(app)
#csrf.init_app(app)

from .proxadmin import proxadmin as proxadmin_blueprint
app.register_blueprint(proxadmin_blueprint)

from .auth import auth as auth_blueprint
app.register_blueprint(auth_blueprint, url_prefix='/auth')

from .uinvoice import uinvoice as uinvoice_blueprint
app.register_blueprint(uinvoice_blueprint, url_prefix='/uinvoice')

@app.errorhandler(403)
def forbidden(e):
    if request.accept_mimetypes.accept_json and \
            not request.accept_mimetypes.accept_html:
        response = jsonify({'error': 'forbidden'})
        response.status_code = 403
        return response
        return render_template('errors/403.html'), 403

@app.errorhandler(404)
def page_not_found(e):
    if request.accept_mimetypes.accept_json and \
            not request.accept_mimetypes.accept_html:
        response = jsonify({'error': 'not found'})
        response.status_code = 404
        return response
    return render_template('errors/404.html'), 404

@app.errorhandler(500)
def internal_server_error(e):
    if request.accept_mimetypes.accept_json and \
            not request.accept_mimetypes.accept_html:
        response = jsonify({'error': 'internal server error'})
        response.status_code = 500
        return response
    return render_template('errors/500.html'), 500

@app.errorhandler(CSRFError)
def handle_csrf_error(e):
    return render_template('errors/csrf_error.html', reason=e.description), 400

@babel.localeselector
def get_locale():
    return request.accept_languages.best_match(app.config['SUPPORTED_LOCALES'])

#@app.before_request
#def before_request():
#    g.request_start_time = time.time()
#    g.request_time = lambda: '%.5fs' % (time.time() - g.request_start_time)
#    g.pjax = 'X-PJAX' in request.headers

if __name__ == '__main__':
    app.run()

